"""
 @file test_dimc_service.py
 @brief An example of using a DIM service.
 This file contains an example for using a DIM service from python. A DimTask
 object is at the same time the DIM service and client.
 Like for the DIM functions there are several steps needed to be taken:
   -# server creates a service
   -# server start 'serving' (dis_start_serving() )
   -# server updates the service
   -# client registers to a service
   -# client wait to be updated

@author Radu Stoica (Radu.Stoica@cern.ch)
\addtogroup examples
@{
"""

import sys
from debug import SAY, ERROR, DEBUG
import debug
debug.DEBUG = 1
from pydim import dis_update_service, dis_add_service, dic_info_service, dis_start_serving, dis_stop_serving
from time import sleep
from threading import Thread, Event

class DimTask:
    """ Creates 3 services and registeres to them. Can be run in multiple instances.
    The class will update "UPDATES" times the services.
    """
    SRV1NAME='Test_Serv1_'
    SRV2NAME='Test_Serv2_'
    SRV3NAME='Test_Serv3_'
    SRV1FORMAT='F:1;I:1;D:2;C:10;C' # buffer size is 35 on 32 bits
    SRV2FORMAT='D'
    SRV3FORMAT='C'
    DELAY=0.03

    def __init__(self, name, clients, updates=10):
        self.name = name
        self.updates = updates
        self.clients = clients
        self.svc1 = dis_add_service(DimTask.SRV1NAME+self.name,
                                    DimTask.SRV1FORMAT,
                                    self.service1, 1)
        self.svc2 = dis_add_service(DimTask.SRV2NAME+self.name,
                                    DimTask.SRV2FORMAT,
                                    self.service2, 2)
        self.svc3 = dis_add_service(DimTask.SRV3NAME+self.name,
                                    DimTask.SRV3FORMAT,
                                    self.service3, 3)
        DEBUG("Service 3 created with ID ", self.svc3)
        DEBUG("DIMTASK %s initialized" %self.name)
        sleep(1)

    def run(self):
        dis_start_serving(self.name)
        print self.clients
        for x in self.clients:
            DEBUG("DIMTASK %s : Registering to service %s"
                  %(self.name, DimTask.SRV1NAME+x))
            dic_info_service(DimTask.SRV1NAME+x, DimTask.SRV1FORMAT,
                                   self.client_callback1)
            DEBUG("DIMTASK %s : Registering to service %s"
                  %(self.name, DimTask.SRV2NAME+x))
            dic_info_service(DimTask.SRV2NAME+x, DimTask.SRV2FORMAT,
                                   self.client_callback2)
            DEBUG("DIMTASK %s : Registering to service %s"
                  %(self.name, DimTask.SRV3NAME+x))
            dic_info_service(DimTask.SRV3NAME+x, DimTask.SRV3FORMAT,
                                   self.client_callback3)
        counter = 0
        DEBUG("DIMTASK %s : Starting service update " %self.name)
        while counter < self.updates:
            counter += 1

            SAY("DIMTASK %s : Updating service nr. 1" %self.name)
            values = (counter, counter+1, 999.0, 999.0 , 'BAU', 'B')
            SAY('DIMTASK %s : Updated %d clients'
                %(self.name, dis_update_service(self.svc1, values)))

            SAY("DIMTASK %s : Updating service nr. 2" %self.name)
            values = (counter, )
            SAY('DIMTASK %s : Updated %d clients' \
                %(self.name, dis_update_service(self.svc2, values)))

            SAY("DIMTASK %s : Updating service nr. 3" %self.name)
            values = ('ALABALAP\x00ORTOCALA', )
            SAY('DIMTASK %s : Updated %d clients' \
                %(self.name, dis_update_service(self.svc3, values)))
            sleep(DimTask.DELAY)
        dis_stop_serving()

    def service1(self):
        return (1, 2, 3.3, 4.4, "A", "B")

    def service2(self):
        return (666.666)

    def service3(self):
        return ("ALABALAPORTOCALA")

    def client_callback1(self, *args):
        SAY('DIMTASK %s : client callback called for service1. Args are %s'
               %(self.name, args))

    def client_callback2(self, *args):
        SAY('DIMTASK %s : client callback called for service2. Args are %s'
               %(self.name, args))

    def client_callback3(self, *args):
        SAY('DIMTASK %s : client callback called for service3. Args are %s'
               %(self.name, args))


def usage():
    print """Usage: python <script name> <DIM Task name> <Other DIM Clients> (comma separated)"""


if __name__=='__main__':
    import sys
    print sys.argv
    try:
        task_name = sys.argv[1]
    except:
        task_name = "task1"
    client_names = sys.argv[2:]
    if not client_names:
        client_names = ("task1",)
    task = DimTask(task_name, client_names, updates=10/DimTask.DELAY)
    task.run()



#!/usr/bin/python
#############################################################################
# Python script to test the PyDimRpc and PyDimRpc classes
#############################################################################

import time
from threading import *
from pydim import DimRpc, DimRpcInfo, dis_start_serving

e = Event()
## 
# @addtogroup examples
# @{
# 
#############################################################################
# Implement some test classes functionality
#############################################################################
class PyRpc (DimRpc):
    def __init__(self):
         DimRpc.__init__(self, 'testRpc', 'I', 'I')
         self.counter = 0

    def rpcHandler(self):
         print('Server side funciton called. Getting parameters...')
         i = self.getInt()
         print("getInt - received: ", i)
         i = self.getData()
         print("getData - received: ", i)
         self.counter += i
         print('Setting response data ', self.counter+1)
         self.setData(self.counter+1) 


class PyRpcInfo(DimRpcInfo):
    def __init__(self):
        DimRpcInfo.__init__(self, 'testRpc', 'I', 'I', None)
        self.value = 1

    def rpcHandler(self):
        print("Non blocking RPC call executed")
        i = getInt()
        print("Received value %d", i)
        
    def run(self):
        print("Calling thread started")
        while not e.isSet():
            print('Calling server function') 
            self.setData(self.value)
            if self.value % 2:
                # blocking   
                print("Getting return value as int: ", self.getInt())
            else:
                # non blocking
                pass
            time.sleep(1)


##
# @}
#

#############################################################################
# Execute tests
#############################################################################

if __name__ == '__main__':
    print("Creating PyDimRpc ...")
    myrpc = PyRpc()
    print("Creating PyDimRpcInfo ...")
    myrpcCaller = PyRpcInfo()

    print("Starting DIM ...")
    dis_start_serving()

    t = Thread(target = myrpcCaller.run)
    t.start()

    try:
        while True:
            time.sleep(1)
    except:
        print("Stopping test")
        e.set()



import os
import sys
from ctypes import *
import glob
lib_folder_path = os.path.dirname(os.path.abspath(__file__))
dimbrowserwrapperlib = cdll.LoadLibrary(
    glob.glob(
        os.path.join(lib_folder_path,"../dimbrowserwrappercpp*.so")
    )[0]
)

__docformat__ = 'reStructuredText'

class DimBrowser(object):
    """This class allows the user to interrogate the DIM DNS."""

    def __init__(self):
        """
        Create new instance of DimBrowser
        """
        self.obj = dimbrowserwrapperlib.DimBrowser_new()

    def __del__(self):
        """
        Destroy DimBrowser instance
        """
        dimbrowserwrapperlib.DimBrowser_delete(self.obj)


    def getServices(self,wildcardServiceName):
        """
        Check if a Service or Services (wildcards allowed) are available.

        :param str wildcardServiceName: The name of a service with or without wildcard (marked with \*)
        :return: The number of services / commands that match the search string.

        The :func:`getNextService()` method can be used to get the type of the services, their names and their format
        """
        return dimbrowserwrapperlib.getServicesFromDNS(self.obj,wildcardServiceName.encode())

    def getNextService(self):
        """
        Has to be called **after** a :func:`getServices()` call.

        :return: This method is a **generator** that generates a tuple that has the following format : *(TypeOfService,NameOfService,FormatOfService)*

        If no service is found, the generator will generate None.
        """
        dimbrowserwrapperlib.getNextServiceFromDNS.restype = py_object
        res = 1
        while res != 0:
            tupleResult = dimbrowserwrapperlib.getNextServiceFromDNS(self.obj)
            if tupleResult == None:
                res = 0
            yield tupleResult

    def getServers(self):
        """
        Get the list of all servers available in the system.

        :return: The number of servers known by the DNS.

        The :func:`getNextServer()` method can be used to retrieve the server names and nodes.
        """
        return dimbrowserwrapperlib.getServersFromDNS(self.obj)


    def getNextServer(self):
        """
        Has to be called **after** a :func:`getServers()` call.

        :return: This method is a **generator** that generates a tuple that has the following format : *(NameOfServer,NodeNameWhereServerRuns)*

        If no server is found, the generator will generate None.
        """
        dimbrowserwrapperlib.getNextServerFromDNS.restype = py_object
        res = 1
        while res != 0:
            tupleResult = dimbrowserwrapperlib.getNextServerFromDNS(self.obj)
            if tupleResult == None:
                res = 0
            yield tupleResult

    def getServerServices(self,serverName):
        """
        Get the list of all services provided by a server.

        :param serverName: The name of the server to get his registered services.
        :return: The number of services provided by the server.

        The :func:`getNextServerService()` method can be used to retrieve the individual services and their formats.
        """
        return dimbrowserwrapperlib.getServerServicesFromDNS(self.obj,serverName.encode())

    def getNextServerService(self):
        """
        Has to be called **after** a :func:`getServerServices` call.

        :returns: This method is a generator that **generates** a tuple that has the following format : *(TypeOfTheCommand,ServiceName,FormatOfService)*

        If no service is found, the generator will return None.
        """
        dimbrowserwrapperlib.getNextServerServiceFromDNS.restype = py_object
        res = 1
        while res != 0:
            tupleResult = dimbrowserwrapperlib.getNextServerServiceFromDNS(self.obj)
            if tupleResult == None:
                res = 0
            yield tupleResult

    def getServerClients(self,serverName):
        """
        Get the list of clients of a server.

        :param serverName: The name of the server to get its connected clients
        :return: The number of clients of the specified server.

        The :func:`getNextServerClient()` can be used to retrieve the individual client process names and nodes.

        **Warning:** The first time this function is called, the correct number of client is returned
        The second time, the number of clients will be increased by one : the first call makes you a client
        of the server !
        """
        return dimbrowserwrapperlib.getServerClientsFromDNS(self.obj,serverName.encode())

    def getNextServerClient(self):
        """
        Has to be called **after** a :func:`getServerClients` call.

        :return: This method is a generator that generates a tuple that has the following format : *(ClientName,NodeNameWhereClientRuns)*

        If no client is found, the generator will return None.
        """
        dimbrowserwrapperlib.getNextServerClientFromDNS.restype = py_object
        res = 1
        while res != 0:
            tupleResult = dimbrowserwrapperlib.getNextServerClientFromDNS(self.obj)
            if tupleResult == None:
                res = 0
            yield tupleResult



import pydim, time

counter = 2
def myDummyFunc():
    global counter 
    counter += 1
    if counter%3 == 1:
         return (1, {1:(1,2,3)}, 'CUCU')
    elif counter%3 == 2:
         return (1, {12312 : ('fdsa',1,2,3), 'fdsa': 434432243})
    else:
        raise Exception('Unimplemented')

if __name__=='__main__':
    pydim.PyDimRpcProxy((myDummyFunc,))
    pydim.dis_start_serving()
    while True:
         time.sleep(1)



#! /usr/bin/python
# ----------------------------------------------------------------------------
#
# -*- coding: iso-8859-1 -*-
#
# debug.py
#
# Logging module
#
# TODO: make everything object oriented and add support for more levels of
# filtering
#
# Needs python > 2.0. Might work with 1.5.X and above

__version__="1.0"
__date__="2006"
__author__="Radu Stoica"

# ----------------------------------------------------------------------------

import sys, string, os

#global _DEBUG, _VERBOSE, _ERROR
_DEBUG   = 0
_VERBOSE = 1
_ERROR   = 1

__private = 'test'

__err = sys.stderr
__out = sys.stdout
__dbg = sys.stdout

def resetErrorStream():
    __err = sys.stderr

def resetOutputStream():
    __out = sys.stdout

def resetDebugStream():
    __dbg = sys.stdout

def setErrorStream(fileDescriptor):
    if not hasattr(fileDescriptor, 'write'):
        raise AttributeError("File descriptor %s has no write method"
                             %fileDescriptor)
    global __err
    __err = fileDescriptor

def setOutputStream(fileDescriptor):
    if not hasattr(fileDescriptor, 'write'):
        raise AttributeError("File descriptor %s has no write method"
                             %fileDescriptor)
    global __out
    __out = fileDescriptor

def setDebugStream(fileDescriptor):
    if not hasattr(fileDescriptor, 'write'):
        raise AttributeError("File descriptor %s has no write method"
                             %fileDescriptor)
    global __dbg
    __dbg = fileDescriptor

def setSyslog(name=None, facility=None):
    import syslog
    class SysLogger:
        def __init__(self, level=syslog.LOG_UUCP):
            self.level = level

        def write(self, msg):
            if msg == '\n': return
            syslog.syslog(self.level, msg)

        def flush(self):
            pass

    if facility is None : facility = syslog.LOG_UUCP
    syslog.openlog(name, 0, facility)
    setOutputStream(SysLogger(syslog.LOG_INFO))
    setErrorStream (SysLogger(syslog.LOG_ERR))
    setDebugStream (SysLogger(syslog.LOG_DEBUG))


def setDebug(value):
    global _DEBUG
    _DEBUG = value

def getCallerID(level, just=20):
    """
    Returns a string containing informations about the function that called it
    at a desired level.
    level = how many level up should the tracing of this function start.

    @param level How many level up should the tracing of this function start.
    @param just: The string will be left justified with the provided input
    @return a string containing informations about the function that called it
    at the desired stack level.
    """
    linePrintStart = ""
    linePrintEnd = ""
    try:
        raise FakeException("")
    except:
        func = sys.exc_info()[2].tb_frame
    while level >= 0:
        func = func.f_back
        level -= 1
    obj = func.f_locals.get("self", None)
    functionName = func.f_code.co_name
    if obj:
        callStr = "%-8s::%-8s %s%3d%s"\
                   %(obj.__class__.__name__,
                    func.f_code.co_name,
                    linePrintStart,
                    func.f_lineno,
                    linePrintEnd)
    else:
        callStr = "%-18s %s%3d%s"\
                  %(func.f_code.co_name, linePrintStart,
                    func.f_lineno, linePrintEnd)
    return callStr.ljust(just)


def LAST_EXCEPTION():
    return str(sys.exc_info()[0]) + " :" + str (sys.exc_info()[1])

def SAY2(*args):
    """Prints messages adding the caller class name and the line of where the call was invoked.
    Behaviour can be changed (on/off) by modifying the _VERBOSE global var.
    """
    if _VERBOSE:
        msg = "%s: %s" %(getCallerID(2), string.join(map(str, args)))
        print(msg, file=__out)
        __out.flush()

def ERROR2(*args):
    """Prints messages adding the caller class name and the line of where the call was invoked.
    Behaviour can be changed (on/off) by modifying the _ERROR global var.
    """
    if _ERROR:
        msg="ERROR: in " + getCallerID(2) + ": " + string.join(map(str, args))
        print(msg, file=__err)
        __err.flush()

def DEBUG2(*args):
    """Prints messages adding the caller class name and the line of where the call was invoked.
    Behaviour can be changed (on/off) by modifying the _DEBUG global var.
    """
    if _DEBUG:
        msg="DEBUG: in " + getCallerID(2) + ": " + string.join(map(str,args))
        print(msg, file=__dbg)
        __dbg.flush()

def SAY(*args):
    """Prints messages adding the caller class name and the line of where the call was invoked.
    Behaviour can be changed (on/off) by modifying the _VERBOSE global var.
    """
    if _VERBOSE:
        msg = "%s: %s" %(getCallerID(1), string.join(map(str, args)))
        print(msg, file=__out)
        __out.flush()

def ERROR(*args):
    """Prints messages adding the caller class name and the line of where the call was invoked.
    Behaviour can be changed (on/off) by modifying the _ERROR global var.
    """
    if _ERROR:
        msg="ERROR: in " + getCallerID(1) + ": ".join(map(str, args))
        print(msg, file=__err)
        __err.flush()

def DEBUG(*args):
    """Prints messages adding the caller class name and the line of where the call was invoked.
    Behaviour can be changed (on/off) by modifying the _DEBUG global var.
    """
    if _DEBUG:
        msg="DEBUG: in " + getCallerID(1) + ": " + string.join(map(str,args))
        print(msg, file=__dbg)
        __dbg.flush()

def parseCommandLine():
    """Sets the debugging location based on the command line arguments given.
    Argument:
        -f (foreground)    -> all logging informations goes to stdout/stderr
        -l (log) <logfile> -> prints to <logfile>
        -s (syslog)        -> prints to syslog using the UUCP facility
    """
    if '-l' in sys.argv[1:]:
        try:
            f = open(sys.argv[sys.argv.index('-l')+1], 'a+')
        except:
            raise RuntimeError("Could not open log file")
        setErrorStream(f)
        setOutputStream(f)
        setDebugStream(f)
        sys.argv.pop(sys.argv.index('-l')+1)
        sys.argv.remove('-l')
    elif '-f' in sys.argv[1:]:
        sys.argv.remove('-f')
    elif '-s' in sys.argv[1:]:
        setSyslog(sys.argv[0])
        sys.argv.remove('-s')
    else:
        setSyslog(sys.argv[0])

def commandLineUsage():

   print("""Output options:
        -f           -> stdout/stderr (foreground mode)
        -l <logfile> -> prints to <logfile>
        -s           -> syslog using the UUCP facility
    """)

# ---------------------------------------------------------------------------
if __name__=='__main__':
    _DEBUG = 1
    SAY("This is a normal output message")
    ERROR("This is an error message")
    DEBUG("This is a debug message")
    print("Sending the same messages through syslog")
    setSyslog("Test debugging")
    SAY("This is a normal output message")
    ERROR("This is an error message")
    DEBUG("This is a debug message")



import sys
from debug import SAY, ERROR, DEBUG
import dimc, debug
from time import sleep
from threading import Thread, Event

debug.DEBUG = 1

SRV1NAME='Test_Serv1'
SRV2NAME='Test_Serv2'
SRV3NAME='Test_Serv3'
SRV1FORMAT='F:1;I:1;D:2;C:10;C:100' # buffer size is 35 on 32 bits
SRV2FORMAT='D:1'
SRV3FORMAT='C:100'

def client_callback1(*args):
    print 'client callback called for service1. Args are', args

def client_callback2(*args):
    print 'client callback called for service2. Args are', args

def client_callback3(*args):
    print 'client callback called for service3. Args are', args

if __name__=='__main__':
    dimc.dic_info_service(SRV1NAME, SRV1FORMAT, client_callback1)
    dimc.dic_info_service(SRV2NAME, SRV2FORMAT, client_callback2)
    dimc.dic_info_service(SRV3NAME, SRV3FORMAT, client_callback3)
    sleep(1000)
    



#!/usr/bin/python
#############################################################################
# Python script to test the PyDimRpc and PyDimRpc classes
#############################################################################

import time, unittest, random
from threading import *
from debug import *

from dimcpp import * 
# importing for the dim_start_serving()
import dimc

##
# @addtogroup examples
# @{
#
class PyRpcInt (DimRpc):
    def __init__(self):
         DimRpc.__init__(self, 'testRpc1', 'I', 'I')
         self.counter = 0

    def rpcHandler(self):
         DEBUG('Getting value')
         i = self.getInt()
         DEBUG("getInt - received: ", i)
         i = self.getData()
         DEBUG("getData - received: ", i)
         self.counter += i
         DEBUG('Setting response data ', self.counter+1)
         self.setData(self.counter+1) 


class PyRpcIntInfo(DimRpcInfo):
    def __init__(self):
        DimRpcInfo.__init__(self, 'testRpc1', 'I', 'I', -1)
        self.value = -1
        self.count = 10

    def rpcInfoHandler(self):
        DEBUG("Getting value")
        i = self.getInt()
        DEBUG("Received value ", i)
        self.value = i
        
# ---------------------------------------------------------------------------
class PyRpcString (DimRpc):
    def __init__(self, name='testRpc2'):
         DimRpc.__init__(self, name, 'C', 'C')
         self.counter = 0

    def rpcHandler(self):
         s = self.getData()
         DEBUG("getData - received: ", s)
         l = [x for x in s]
         l.sort()
         s = ""
         for x in l: s += str(x)
         DEBUG('Setting response data ', s)
         self.setData(s)

class PyRpcStringInfo(DimRpcInfo):
    def __init__(self, name='testRpc2'):
        DimRpcInfo.__init__(self, name, 'C', 'C', 'Error')
        self.value = 'Error'

    def rpcInfoHandler(self):
        s = self.getString()
        DEBUG("Received value '"+s+"'")
        self.value = s

# ---------------------------------------------------------------------------
class PyRpcComposite (DimRpc):
    def __init__(self):
        self.format_in = 'I:1;F:2;D:2;X:1;C'
        self.format_out = 'C:10;I'
        DimRpc.__init__(self, 'testRpc3', self.format_in, self.format_out)
        self.counter = 0

    def rpcHandler(self):
         s = self.getData()
         DEBUG("getData - received: ", s)         
         tup = ("It's OK123", 1, 2, 3, 4, 5, 6, 7, 8, 9)
         DEBUG('Setting response data: ', tup)
         self.setData("It's OK123", 1, 2, 3, 4, 5, 6, 7, 8, 9)
         #self.setData(tup)

class PyRpcCompositeInfo(DimRpcInfo):
    def __init__(self):
        self.format_in = 'I:1;F:2;D:2;X:1;C'
        self.format_out = 'C:10;I'
        DimRpcInfo.__init__(self, 'testRpc3', self.format_in, 
                            self.format_out, None)
        self.value = None 

    def rpcInfoHandler(self):
        v = self.getData()
        DEBUG("Received value ", v)
        self.value = v

#############################################################################
# Implements test classes functionality
#############################################################################
myRpcInt = PyRpcInt()
myRpcIntCaller = PyRpcIntInfo()
myRpcString = PyRpcString()
myRpcStringCaller = PyRpcStringInfo()
myRpcComposite = PyRpcComposite()
myRpcCompositeCaller = PyRpcCompositeInfo()
#myRpcString = PyRpcString()
myRpcFailCaller = PyRpcStringInfo('testRcp4')
DEBUG("Starting DIM")
dimc.dic_disable_padding()
dimc.dis_start_serving()
dimc.dic_disable_padding()
time.sleep(1)

class TestDimRpc(unittest.TestCase):

    def testRpcInt(self):
        print('\n'+80*'-')
        print('Running RPC int test')
        print(80*'-')
        DEBUG('Running RPC int test')
        time.sleep(1)
        count = 1
        res = 0
        for i in range(1,10):
            count += i
            myRpcIntCaller.setData(i)
            res = myRpcIntCaller.getData()
            self.assertEqual(count, res) 
     
    def testRpcString(self):
        print('\n'+80*'-')
        print('Running RPC string test')
        print(80*'-'+'\n')
        time.sleep(1) # dim threads are not initialized imediately unfortunately 
        r=range(1,10)
        s1 = ""
        s2 = ""
        for x in r: s1 += str(x)
        random.shuffle(r)
        for x in r: s2 += str(x)
        myRpcStringCaller.setData(s2)
        res1 = myRpcStringCaller.getData() 
        res2 = myRpcStringCaller.value  
        self.assertEqual(s1, res1)
        self.assertEqual(s1, res2)
        
    def testRpcComposite(self):
        print('\n'+80*'-')
        print('Running RPC test using composite parameters')
        print(80*'-')
        tup = (1, 2.1, 2.2, 3.1, 3.2, 1234567890123, "Test string")
        DEBUG("Setting data", tup)
        myRpcCompositeCaller.setData(1, 2.1, 2.2, 3.1, 3.2, 1234567890123, "Test string")
        res1 = myRpcCompositeCaller.getData() 
        res2 = myRpcCompositeCaller.value
        DEBUG(res2)
        tup=("It's OK123", 1, 2, 3, 4, 5, 6, 7, 8, 9)
        self.assertEqual(tup, res1)
        self.assertEqual(tup, res2)

    def testRpcFailure(self):
        print('\n'+80*'-')
        print('Running RPC test using composite parameters')
        print(80*'-')
        myRpcFailCaller.setData('ALA BALA PORTOCALA')
        res1 = myRpcFailCaller.value
        res2 = myRpcFailCaller.getData()
        DEBUG(myRpcFailCaller.value)
        self.assertEqual('Error', res1)
        self.assertEqual(res1, res2)

##
# @}
#

#def suite():
#    tests = ['testRpcInt', 'testRpcString', 'testRpcComposite']
#    return unittest.TestSuite(map(WidgetTestCase, tests))


#############################################################################
# Execute tests
#############################################################################

if __name__ == '__main__':
    unittest.main()




"""
@file test_dimc_cmnd.py
@brief An example of using a command service from Python.
Contains both the server and client parts.

@author Radu Stoica (Radu.Stoica@cern.ch)
\addtogroup examples
@{
"""

from debug import SAY, ERROR, DEBUG
import sys, pydim, debug
from time import sleep
import gc

debug._DEBUG = 1


COMMAND_DELAY=3

CMND1FORMAT="C"
CMND2FORMAT="F:2;D:2;I:1;S:1;C"
CMND3FORMAT="I"

def dummy(*args):
  SAY('Server: I am an unbound dummy function. I\'ve received', args)


class MyClass:
  def dummy(self, *args):
    SAY("Server: I am dummy func bound to a 'MyClass' instance I\'ve received", args)


def server_create_command_services():
  pydim.dis_add_cmnd('test1', CMND1FORMAT, dummy, 1)
  SAY('Added command test1 with DIM format %s' %CMND1FORMAT)

  pydim.dis_add_cmnd('test2', CMND2FORMAT, MyClass().dummy, 2)
  SAY('Added command test2 with DIM format %s' %CMND2FORMAT)

  pydim.dis_add_cmnd('test3', CMND3FORMAT, dummy, 2)
  SAY('Added command test3 with DIM format %s' %CMND3FORMAT)

  pydim.dis_start_serving()


if __name__ == '__main__':
  # Server configuration:
  server_create_command_services()

  # The client command execution
  counter = 0
  while  counter<1000 or True:
    counter += 1
    # executing the first command
    print ""
    tuple_args = ('Test call no. %03d' %counter, )
    res = pydim.dic_cmnd_service('test1', tuple_args, CMND1FORMAT)
    if res:
      SAY('Client: Successfully executed command test1 (return code %d)' %res)
    else:
      SAY('Client: Command test1 execution failed (return code %d)' %res)
    sleep(COMMAND_DELAY)

    # second command
    print ""
    tuple_args = (counter, counter+1, counter*9, counter*1.9, counter, counter-1, 'ABRACADABRA')
    res = pydim.dic_cmnd_service('test2', tuple_args, CMND2FORMAT)
    if res:
      SAY('Client: Successfully executed command test2 (return code %d)' %res)
    else:
      SAY('Client: Command test2 execution failed (return code %d)' %res)
    sleep(COMMAND_DELAY)
  
    # last command
    print ""
    list_args = range(counter, counter+2) # lists are also accepted
    res = pydim.dic_cmnd_service('test3', list_args, CMND3FORMAT)
    if res:
      SAY('Client: Successfully executed command test3 (return code %d)' %res)
    else:
      SAY('Client: Command test3 execution failed (return code %d)' %res)

    sleep(COMMAND_DELAY)



#! /usr/bin/env python

"""
PyDIM is a Python interface to DIM.

PyDIM could be used to create DIM clients and servers, using an API very similar
to the one that is used for C.
"""

import random
import string
import time
import types
import threading
import logging

from dimc import *
from dimcpp import *
from .debug import *

_version = '1.3.4'

# ###########################################################################
# The DIM RPC proxy class.
# ###########################################################################
class PyDimRpcProxy (DimRpc):
    """
Class that facilitates publishing of random python functions using a single
DIM RPC. At creation it receives a lost of callable objects wo which the calls
will be passed. It creates a single string DIM RPC and accepts a string with
the format:
  'function_name/par1_name=par1_value1,par1_value2/par2_name=par2_value/.../'
or
  'function_name/par1_value,par2_value/.../' for possitional arguments
The appropiate function is called with the appropiate parameters and the
result is returned in the same format.
The special characters (',', '=' and '/') must be excaped.
Identifiers must contain alfanumeric characters plus the '_' and '-' chars.
It makes the assumption that the python functions will return always a tuple
in the format (STATUSCODE, RESULTS). All the return parameters are converted
to a string and are returned to the client.
    """
    def __init__(self, funcs, rpcName='testRPC'):
        DimRpc.__init__(self, rpcName, 'C', 'C')
        self.lastValue = None
        self.funcs = {}
        self.name = rpcName
        if not hasattr(funcs, '__iter__'):
            funcs = [funcs]
        for f in funcs:
            # creating a dictionary of function names and callable objects
            if hasattr(f, '__call__'):
                self.funcs[f.__name__] = f
            else:
                ERROR ('Object %s is not callable' %f)
                DEBUG(dir(f))

    def convert(self, args):
        ret = ""
        if not hasattr(args, '__iter__'):
            args = (args,)
        for x in args:
            add = str(x)
            add = add.replace('/', '\/')
            add = add.replace('=', '\=')
            add = add.replace(',', '\,')
            ret += add + ','
        return ret[:-1]

    def split(self, s, sep, exc='\\'):
        args = []
        poz = 0
        while s:
            poz = s.find(sep, poz)
            if poz == 0:
                s = s[1:]
            elif poz == -1:
                args.append(s)
                s = ""
            elif s[poz-1] != exc:
                args.append(s[:poz])
                s = s[poz+1:]
                poz = 0
            else:
                poz += 1
        if len(args) > 1 or len(args) == 0:
            return args
        else:
            return args[0]

    def parse(self, s):
        pozArgs = []
        keyArgs = {}
        args = self.split(s, '/')
        if type(args) is str: args = (args,)
        DEBUG(args)
        for arg in args:
            #DEBUG(arg)
            poz = arg.find('=')
            if poz > 0 and arg[poz-1] != '\\':
                # we have a named argument
                keyArgs[arg[:poz]] = self.split(arg[poz+1:], ',')
            else:
                pozArgs.append(self.split(arg, ','))
        return pozArgs, keyArgs

    def rpcHandler(self):
        s = self.getString()
        DEBUG("Received: ", s)
        # figuring out which method to call and parsing arguments
        if s.find('/') > -1:
            funcName = s[:s.find('/')]
            s = s[s.find('/'):]
        elif s:
            # we have 0 arguments
            funcName = s
            s = ""
        else:
            self.setData('ERROR: function %s is not registered' %s)
            return
        try:
            funcObj = self.funcs[funcName]
        except KeyError as e:
            ERROR(e)
            self.setData('status=FAIL/error=Could not find function '+str(e))
            return
        #DEBUG(funcName, s)
        pozArgs, keyArgs = self.parse(s)
        #DEBUG(pozArgs, keyArgs)
        ret = funcName + '/'

        try:
           res = funcObj(*pozArgs, **keyArgs)
        except Exception as e:
            # catch all convert it to string and return
            print(type(funcObj))
            ret += 'status=FAIL/error='+str(e)
            DEBUG('Returning exception message: %s' %ret)
        else:
            '''
            We have an result. Converting to a string and returning it.
            There can be two types of returned results: iterables and
            dictionaries. Both results can contain basic types or other
            iterables.
            Iterables: They will be  converted in the equivalent of positional
            arguments for calling a function.
            Dictionaries: Will be converted in the equivalent of named
            arguments.
            '''
            if res[0] == 0:
                ret += 'status=ERROR/'
                try:
                    ret = '%serror=%s/' %(ret, res[1])
                except:
                    pass
            else:
                ret += 'status=SUCCESS/'
                named = False
                try:
                    res = res[1]
                except:
                    pass
                else:
                    if isinstance(res, dict):
                        for x in res:
                            ret = "%s%s=%s/" %(ret,
                                               self.convert(x),
                                               self.convert(res[x])
                                               )
                    elif hasattr(res, '__iter__'):
                        for x in res:
                            ret = "%s%s/" %(ret, self.convert(x))
                    else:
                        ret = "%s%s/" %(ret, self.convert(res))
	    # this is particularly usefull if on the other side are C/C++ clients
        ret += '\0'
        # setting result
        DEBUG('Sending result: %s' %ret)
        self.setData(ret)



# ###########################################################################
# The DIM DNS abstraction class
# ###########################################################################
class Dns:
    def __init__(self):
        self.dns_srv_sub = dim.Subscription(name = "DIS_DNS/SERVER_LIST",
                                            tag = self, schema = "S",
                                            handler = self.servers_update)
        self.servers = None
    def services_update(self, svcstr):
        self.services = []
        for s in string.split(svcstr[0], '\n'):
            if s == '': continue
            i = string.rfind(s, '|')
            j = string.rfind(s, '|', 0, i)
            print(s, i, j)
            if (i == len(s) - 1):
                type = "SVC"
            else:
                type = s[i + 1:(len(s))]
            self.services.append([ s[0:j], s[j+1:i], type])
        self.service_update = True

    def servers_update(self, srv):
        self.servers = []; self.server_host = []
        for s in string.split(srv[0], '|'):
            [server, host ] = string.split(s, '@')
            self.servers.append(server)
            self.server_host.append([server, host])

    def get_services(self, server):
        if (not server in self.servers): return None;
        self.service_update = False
        srv = dim.Subscription(name=server+"/SERVICE_LIST", schema = "S",
                               handler = self.services_update)
        i = 0
        while (self.service_update == False and i < 50):
            i += 1
            time.sleep(0.1)
        return self.services
    def get_servers(self):
        while (self.servers == None):
            time.sleep(0.1)
        return self.servers

def dim_service(fn):
    """
    A decorator function that makes easier to use normal functions as callbacks
    for DIM services.

    'fn' is a function (in general, a callable object) that doesn't
    receive arguments, and returns a function that is suited for DIM callbacks.
    """
    def svc(tag):
        # Tag argument is declared but not used
        rtn = fn()
        # The returned value must be tuple, so we try to convert simple values
        if not rtn in (list, tuple):
            rtn = (rtn,)
        return rtn

    return svc

def dim_service_tag(fn):
    """
    A decorator function that makes easier to use function with an argument as
    callbacks for DIM services.

    'fn' is a function (in general, a callable object) that receives one
    argument, and returns a function that is suited for DIM callbacks. The
    argument used is the tag of the service tag registerd with DIM.
    """
    def svc(tag):
        rtn = fn(tag)
        if not rtn in (list, tuple):
            rtn = (rtn,)
        return rtn

    return svc


def dic_sync_info_service(name, description=None, timeout=None, default_value=None):
    """
    A synchronous call for getting the value of a service.

    This function is equivalent to `dic_info_service`, but it waits until the
    value is retrieved and returns the value of the service. For this reason a
    callback function is not needed. Warning: If the service is not available,
    it will not wait for the value and this function will return an empty tuple.

    Arguments are:

       *name* Service name. same name used by server when declaring the service.

       *description* The description string of the service. Optional, default value is None.

       *timeout* The number of seconds after which the service is considered to
        have failed. Optional, by default there is no timeout.

       *default_value* The value that will be returned in case the service
        doesn't succeed. Optional, default is `None`.
    """
    executed = threading.Event()
    state = dict(value=None)

    def create_callback(st):
        def _callback(*args):
            st['value'] = args
            executed.set()

        return _callback

    callback = create_callback(state)

    if timeout is None:
        dim_timeout = 0
    else:
        dim_timeout = timeout
    tag = random.randint(0,100000)
    if description is None:
        """Description is not provided in the dic_sync_info_service function"""
        sid = dic_info_service(name, callback, ONCE_ONLY, dim_timeout, tag, default_value)
    else:
        sid = dic_info_service(name, description, callback, ONCE_ONLY, dim_timeout, tag, default_value)

    executed.wait(timeout)

    dic_release_service(sid)

    return state['value']

def dic_sync_cmnd_service(name, arguments, description=None, timeout=None):
    """
    A synchronous call for executing a command.

    This function works in the same way as `dic_cmnd_service`, but it waits
    until the command is executed.

    Arguments are:

       *name* Command name, same name used by server when declaring the command
       service.

       *arguments* A tuple with the values that are sent to the command as
       arguments.

       *description* The description string of the command. Optional, default value is None.
       If the description is **None**, pydim will do a DNS call to get the description string expected
       by the command. After the first call, the description will be stored in a **cache** that can
       be enabled or disabled. For more informations about that, see `DNS cache`_.

       *timeout* The number of seconds after which the command is considered
       to have failed. Optional, by default there is no timeout.

    It returns an integer which indicates if the command was executed
    successfully; 1 if it was correctly sent to the server, 0 otherwise.

    """
    executed = threading.Event()
    state = dict(retcode=None)

    def create_callback(st):
        def _callback(tag, retcode):
            state['retcode'] = retcode
            executed.set()

        return _callback

    callback = create_callback(state)
    tag = 0
    if description is None:
        dic_cmnd_callback(name, arguments, callback, tag)
    else:
        dic_cmnd_callback(name, arguments, description, callback, tag)

    executed.wait(timeout)

    return state['retcode']


if __name__ == "__main__":
    # in case the file is executed directly
    dns = Dns()
    print(dns.get_servers())
    pass



'''
\brief This is the configuration file that is also used to build the package.
It uses the standard distutils Python module to provide a platform independent
build mechanism
'''
from setuptools import setup, Extension
import setuptools
import os, sys
import os.path as path

def getOS():
  if os.getenv('OS'):
      os_name = os.getenv('OS')
      '''Building DIM depends on $OS. If this is wrongly set then building DIM
  doesn't work and the installation will fail...
      '''
  else:
      os_name = os.uname()[0].lower()
  return os_name

OS = getOS()

LIBRARIES = ['dim']
#
# Use the following variable if you need to pass any special options to the
# compiler.
#
COMPILE_ARGS = []
if OS.lower().find('win') > -1:
    COMPILE_ARGS += ['-DWIN32','-D__DEBUG', '-DDEBUG']
    LIBRARIES.append('ws2_32')
else:
    LIBRARIES.append('dl')
    COMPILE_ARGS += [ '-Wno-deprecated']
#    COMPILE_ARGS += ['-g', '-O0', '-D__DEBUG', '-DDEBUG', '-fno-inline', '-fno-default-inline']


# This can be used to build in debug mode and/or profiling. The internal debug
# of the DIM wrapper can be turned on using the -DDEBUG. For example:
#
#
# Additional compile flags can be set using the CFLAGS environment variable or
# by creating a Setup file with the complementary instructions:
#
# http://docs.python.org/install/index.html#tweaking-compiler-linker-flags
#

# ###########################################################################
# Although we should already have all the build info by this point we can
# still do a bit of guess work to find the DIM location(s). Building DIM
# usually creates some environment variables that we can search for.

# The expected location for the DIM.
dim_dir = None

# The expected location for the DIM headers is in DIM_HOME/dim.
dim_include_dirs = []

# The location where the DIM library (libdim.so on linux or dim.dll on Windows)
dim_library_dirs = []

# The following environment variables will be used looking for the Dim installation
dim_env_vars = ('DIMDIR', 'DIMHOME', 'DIM_HOME')
for dim_var in dim_env_vars:
    if os.getenv(dim_var):
        dim_dir = os.getenv(dim_var)
        dim_include_dirs = [dim_dir, os.path.join(dim_dir, 'dim')]
        dim_library_dirs = [dim_dir, os.path.join(dim_dir, 'bin'),os.path.join(dim_dir, OS)]
        break


if dim_include_dirs == [] and os.sys.platform == 'linux':
    dim_include_dirs = [ '/usr/local/include/dim', '/usr/include/dim' ]
if dim_library_dirs == [] and os.sys.platform == 'linux':
    if os.uname()[4] == 'x86_64':
        dim_library_dirs = [ '/usr/local/lib64/' ]
    else:
        dim_library_dirs = [ '/usr/local/lib' ]

cwd = os.getcwd()
include_dirs = dim_include_dirs
include_dirs.append(os.path.join(cwd, 'src'))
include_dirs.append(os.path.join(cwd, 'dimbrowser'))
library_dirs = dim_library_dirs
compile_args = COMPILE_ARGS
libraries = LIBRARIES
platform = sys.platform
version = open('./VERSION').read().strip()

#Printing the final variables and building the package
print(80*'-')
print('Welcome to the DIM Python interface module (PyDIM) installer.')
print(80*'-')
print('Using variables:')
print('DIM dirs: %s' % dim_dir)
print('OS: %s' % OS)
print('Include dirs: %s' %include_dirs)
print('Library dirs: %s' %library_dirs)
print('Compile args: %s' %compile_args)
print(80*'-')

dimmodule = Extension('dimc',
                      sources = ['src/dimmodule.cpp','src/pydim_utils.cpp'],
                      include_dirs = include_dirs,
                      libraries = libraries,
                      library_dirs = library_dirs,
                      extra_compile_args = compile_args
                     )
dimmodule_cpp = Extension('dimcpp',
                      sources = ['src/dimcppmodule.cpp', 'src/pydim_utils.cpp'],
                      include_dirs = include_dirs,
                      libraries = libraries,
                      library_dirs = library_dirs,
                      extra_compile_args = compile_args
                      )
dimbrowsermodule = Extension('dimbrowserwrappercpp',
                      sources = ['dimbrowser/dimbrowserwrapper.cpp'],
                      include_dirs = include_dirs,
                      libraries = libraries,
                      library_dirs = library_dirs,
                      extra_compile_args = compile_args
                      )

with open("README.md","r") as fh:
    long_description = fh.read()

pydim_classifiers = [
    "Programming Language :: Cython"
]

setup(name = 'pydim3',
      version = version,
    #   description = 'Python interface package for DIM C and C++ interfaces.',
      long_description_content_type='text/markdown',
      long_description = long_description,
    #   license='GPL',
    #   url = 'http://lhcbdoc.web.cern.ch/lhcbdoc/pydim/index.html',
    #   author = 'Niko Neufeld (originally by Radu Stoica)',
    #   author_email = 'niko.neufeld@cern.ch',
      py_modules = ['pydim/__init__', 'pydim/debug','dimbrowser/__init__','dimbrowser/dimbrowser'],
      ext_modules = [dimmodule_cpp,dimbrowsermodule,dimmodule],
      packages=setuptools.find_packages(),
      scripts = [],
    #   python_requires='>=3.0',
    #   classifiers = pydim_classifiers
      )




from pydim.debug import SAY, ERROR, DEBUG
import sys, pydim, pydim.debug as debug
from time import sleep
import gc

debug._DEBUG = 1

CMND1FORMAT="C"
CMND2FORMAT="F:2;D:2;I:1;S:1;C"
CMND3FORMAT="I"
COMMAND_DELAY = 1

def dummy(*args):
    SAY('Server: I am an unbound dummy function. I\'ve received', args)


class MyClass:
    def dummy(*args):
        SAY("Server: I am dummy func bound to a 'MyClass' instance. I\'ve received", args)


def client_callback(tag, ret):
    SAY('The client callback function has received tag %d and return code %d' %(tag, ret))


def server_create_command_services():
    #adding the first command
    pydim.dis_add_cmnd('test1', CMND1FORMAT, dummy, 1)
    SAY('Added command test1 with DIM format %s' %CMND1FORMAT)
    #adding the second command
    pydim.dis_add_cmnd('test2', CMND2FORMAT, dummy, 2)
    SAY('Added command test2 with DIM format %s' %CMND2FORMAT)
    #adding the third command
    pydim.dis_add_cmnd('test3', CMND3FORMAT, dummy, 2)
    SAY('Added command test3 with DIM format %s' %CMND3FORMAT)
    #starting DIM
    pydim.dis_start_serving()


if __name__ == '__main__':
    # Server configuration:
    server_create_command_services()

    #The client command execution
    counter = 0
    while  counter<1000 or True:
        counter += 1
        print('\n\n'+80*'-')
        print('Iteration number %d' %counter)
        # executing the first command
        tuple_args = ('Test call no. %03d' %counter, )
        res = pydim.dic_cmnd_callback('test1', tuple_args, client_callback, 1)
        if res:
            SAY('Client: Successfully executed command test1 (return code %d)' %res)
        else:
            SAY('Client: Command test1 execution failed (return code %d)' %res)
        sleep(COMMAND_DELAY)
        print

        # second command
        tuple_args = (counter, counter+1, counter*9, counter*1.9, counter, counter-1, 'ABRACADABRA')
        res = pydim.dic_cmnd_callback('test2', tuple_args, client_callback, 2)
        if res:
            SAY('Client: Successfully executed command test2 (return code %d)' %res)
        else:
            SAY('Client: Command test2 execution failed (return code %d)' %res)
        sleep(COMMAND_DELAY)
        print

        # last command
        list_args = range(counter, counter+5) #also lists are accepted
        res = pydim.dic_cmnd_callback('test3', list_args, client_callback, 3)
        if res:
            SAY('Client: Successfully executed command test3 (return code %d)' %res)
        else:
            SAY('Client: Command test3 execution failed (return code %d)' %res)
        sleep(COMMAND_DELAY)
        print(80*'-')



import sys
import pydim
from pydim import debug
from pydim.debug import SAY, ERROR, DEBUG
from time import sleep

debug._DEBUG = 1


global counter
counter = 1

def dummy(*args):
    print('I am an unbound dummy func', args)

def dis_service1 (tag):
    print('function servce1 called')
    global counter
    return (counter, counter+1, 10.0, 10.0 , 'CUCU', 'A')

class Struct:
    def dummy(self, *args):
        print('function servce2 called')
        print("I am dummy func bound to Struct. Received", args)

    def service2(self, tag):
        global counter
        counter += 1
        return (counter,)


print(dir(pydim))


def test_dis_interface():
    print(80*'-')
    x = pydim.dis_get_dns_node()
    pydim.dis_set_dns_node('tralala')
    if not pydim.dis_get_dns_node() == 'tralala':
        ERORR('get/set dns failed')
        sys.exit(1)
    pydim.dis_set_dns_node(x)
    print('dis_get/dns_node functions tested')

    print(80*'-')
    x = pydim.dis_get_dns_port()
    pydim.dis_set_dns_port(-2525)
    if not pydim.dis_get_dns_port() == 2525:
        ERROR('get/set dns port failed. Received ports',x,pydim.dis_get_dns_port() )
    pydim.dis_set_dns_port(x)
    print('dis_get/dns_port functions tested')

    s=Struct()

    pydim.dis_add_exit_handler(dummy)
    pydim.dis_add_exit_handler(s.dummy)

    pydim.dis_add_error_handler(dummy)
    pydim.dis_add_error_handler(s.dummy)

    pydim.dis_add_client_exit_handler(dummy)
    pydim.dis_add_client_exit_handler(s.dummy)

    pydim.dis_start_serving()
    #pydim.dis_stop_serving()
    svc1 = pydim.dis_add_service('Test Service Nr.1', "F:1;I:1;D:2;C:1;C:1;", dis_service1, 1)
    svc2 = pydim.dis_add_service('Test Service Nr.2', "D:1", Struct().service2, 2)

    pydim.dis_add_cmnd('test1', "C:20", dummy, 1)
    pydim.dis_add_cmnd('test2', "F:2;D:2;I:3;S:1;C:5;C:1", Struct().dummy, 2)

    print('dis_update_service', pydim.dis_update_service(svc1))

    print('dis_selective_update_service', pydim.dis_selective_update_service(svc1,(1,2,3)))
    print('dis_selective_update_service', pydim.dis_selective_update_service(svc1,[1,2,3]))

    print('dis_set_quality', pydim.dis_set_quality(svc1, 10))
    print('dis_set_timestamp: ', pydim.dis_set_timestamp(svc1,1,1))
    print('dis_remove_service: ',pydim.dis_remove_service(svc1))

    print('dis_get_next_cmnd: ',pydim.dis_get_next_cmnd(10))
    print('dis_get_client: ', pydim.dis_get_client('fdas'))
    print('dis_get_conn_id: ', pydim.dis_get_conn_id())
    print('dis_get_timeout: ', pydim.dis_get_timeout(svc1, 1))
    print('dis_get_client_services: ', pydim.dis_get_client_services(10))
    print('dis_set_client_exit_handler', pydim.dis_set_client_exit_handler(1,1))
    print('dis_get_error_services', pydim.dis_get_error_services())
    i=1
    sleep(1)

    print(("Starting serving services 1 and 2. Their ids are", svc1, svc2))
    pydim.dis_start_serving()
    while True:
         global counter
         print("counter = ",counter)
         if counter%2:
             counter += 1
             print("Updating service nr. 1")
             print(pydim.dis_update_service( svc1, (counter, counter+1, 999.0, 999.0 , 'BAU', 'B') ))
             print("Updating service nr. 2 counter%2")
             print(pydim.dis_update_service(svc2, (100.0,) ))
         else:
             print("Updating service nr. 1")
             print(pydim.dis_update_service( svc1 ))
             print("Updating service nr. 2")
             print(pydim.dis_update_service( svc2 ))
         sleep(5)

global __dic_service_not_refreshed
__dic_service_refreshed = 0

def dic_command_callback(*args):
    print('dic callback called. Args are', args)
    global __dic_service_refreshed
    __dic_service_refreshed = 1

def test_dic_interface():
    pydim.dic_info_service('SERVICE_NO_FUNC', 'C:1;F:1;C:20', dic_command_callback)
    while not __dic_service_refreshed:
       sleep(1)

if __name__=='__main__':
    test_dis_interface()
    



import unittest
import pydim
from dimbrowser import DimBrowser
import time
import threading
import os
import socket

SERVER_NAME="server_test_dimbrowser"
FIRST_SERVICE_NAME="say_hello_service"
SECOND_SERVICE_NAME="say_hi_service"
FIRST_SERVICE_FORMAT = "C"
SECOND_SERVICE_FORMAT = "C"

COMMAND_1_NAME="example-command-1"
COMMAND_1_FORMAT="D"

def get_hostname():
    return socket.getfqdn()


class Server(threading.Thread):
    '''
    Represents a DIM server
    '''
    def __init__(self):
        threading.Thread.__init__(self)
        self.isStopped = False
        self.createSayHelloService()
        self.createSayHiService()
        self.startServer()

    def say_hello_service_callback(self,tag):
        return ("Hello",)

    def say_hi_service_callback(self,tag):
        return("Hi",)

    def createSayHelloService(self):
        self.sayHelloService = pydim.dis_add_service(FIRST_SERVICE_NAME,FIRST_SERVICE_FORMAT,self.say_hello_service_callback,0)
        if not self.sayHelloService:
            #print("%s has not been created" % (FIRST_SERVICE_NAME))
            sys.exit(1)

        #print("service %s has been registered" % FIRST_SERVICE_NAME)
        pydim.dis_update_service(self.sayHelloService)

    def createSayHiService(self):
        self.sayHiService = pydim.dis_add_service(SECOND_SERVICE_NAME,SECOND_SERVICE_FORMAT,self.say_hi_service_callback,0)
        if not self.sayHiService:
            sys.exit(1)

        pydim.dis_update_service(self.sayHiService)

    def example1Callback(value,tag):
        print("%s call received, value = %s" % (COMMAND_1_NAME,value[0]))

    def addExampleCmnd1(self):
        self.exampleCommand1 = pydim.dis_add_cmnd(COMMAND_1_NAME,COMMAND_1_FORMAT,self.example1Callback,1)

    def removeExampleCmnd1(self):
        pydim.dis_remove_service(self.exampleCommand1)

    def startServer(self):
        #print("Starting %s..." % SERVER_NAME)
        pydim.dis_start_serving(SERVER_NAME)
        #print("%s started" % FIRST_SERVICE_NAME)

    def stop(self):
        pydim.dis_remove_service(self.sayHelloService)
        pydim.dis_remove_service(self.sayHiService)
        self.isStopped = True

    def run(self):
        while not self.isStopped:
            pass

class Client(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.isStopped = False

    def stop(self):
        self.isStopped = True

    def firstServiceCallback(self,value):
        pass

    def run(self):
        res1 = pydim.dic_info_service(FIRST_SERVICE_NAME, self.firstServiceCallback)
        while not self.isStopped:
            pass

class DimBrowserTestCase(unittest.TestCase):
    def setUp(self):
        self.dbr = DimBrowser()

    @classmethod
    def setUpClass(cls):
        cls._server = Server()
        cls._server.start()

    @classmethod
    def tearDownClass(cls):
        cls._server.stop()
        del cls._server

    def tearDown(self):
        del self.dbr

    def test_DimBrowser_instanciation(self):
        self.assertNotEqual(self.dbr,None,'DimBrowser is not instanciated')

class DimBrowserGetServicesTestCase(DimBrowserTestCase):
    def test_one_service_returned_by_getServices(self):
        self.assertEqual(self.dbr.getServices(FIRST_SERVICE_NAME),1)

    def test_first_name_of_service_returned_is_correct(self):
        self.dbr.getServices(FIRST_SERVICE_NAME)
        result = next(self.dbr.getNextService())
        self.assertIsNotNone(result)
        self.assertEqual(result[1],FIRST_SERVICE_NAME)
        self.assertEqual(result[2],FIRST_SERVICE_FORMAT)

    def test_0_service_returned_when_no_service_found(self):
        self.assertEqual(self.dbr.getServices("Service_that_doesn_t_exists"),0)

    def test_none_returned_when_no_service_found(self):
        self.dbr.getServices("Service_that_doesn_t_exists")
        result = next(self.dbr.getNextService())
        self.assertEqual(result,None)

    def test_multiple_services_return_when_wildcard(self):
        service_name = "say_h*"
        nb_services = self.dbr.getServices(service_name)
        self.assertGreater(nb_services,1)
        compteur = 0
        for res in self.dbr.getNextService():
            if res != None:
                compteur+=1
        self.assertEqual(compteur,2)

    def test_format_is_correct_say_hello_service(self):
        service_name = FIRST_SERVICE_NAME
        self.dbr.getServices(service_name)
        res  = next(self.dbr.getNextService())
        self.assertEqual(res[2],FIRST_SERVICE_FORMAT)

class DimBrowserGetServersTestCase(DimBrowserTestCase):

    def test_get_servers(self):
        nb_servers = self.dbr.getServers()
        self.assertEqual(nb_servers,2)

    def test_get_next_server_contains_SERVER_NAME_and_node(self):
        self.dbr.getServers()
        contains_server_name_and_node = False
        res = ("",)
        while res is not None and not contains_server_name_and_node:
            if res[0] == SERVER_NAME:
                if res[1] == get_hostname():
                    contains_server_name_and_node = True
            res = next(self.dbr.getNextServer())

        self.assertTrue(contains_server_name_and_node)


class DimBrowserGetServerServicesTestCase(DimBrowserTestCase):

    def test_number_of_services_returned_getServerServices(self):
        nbServices = self.dbr.getServerServices(SERVER_NAME)
        self.assertEqual(nbServices,2 + 5) # 2 created services + 5 default services

    def test_number_of_services_0_when_server_does_not_exists(self):
        nbServices = self.dbr.getServerServices("server_not_exists")
        self.assertEqual(nbServices,0)

    def test_get_next_server_service_returns_none_when_server_does_not_exists(self):
        self.dbr.getServerServices("server_not_exists")
        self.assertIsNone(next(self.dbr.getNextServerService()))

    def test_get_next_server_service_contains_services_created(self):
        service_1_found = False
        service_2_found = False
        self.dbr.getServerServices(SERVER_NAME)
        res = ("","")
        while res is not None and not (service_1_found and service_2_found):
            res = next(self.dbr.getNextServerService())
            if res is not None:
                if res[1] == FIRST_SERVICE_NAME:
                    service_1_found = True
                elif res[1] == SECOND_SERVICE_NAME:
                    service_2_found = True

        self.assertTrue(service_1_found)
        self.assertTrue(service_2_found)

class DimBrowserGetServerClientsTestCase(DimBrowserTestCase):
    '''GetServerClient cannot be tested efficiently'''

    def test_no_client_connected_to_server(self):
        nbClientsConnected = self.dbr.getServerClients(SERVER_NAME)
        self.assertEqual(nbClientsConnected,0)
        self.assertIsNone(next(self.dbr.getNextServerClient()))

    def test_z_client_connected_to_server(self):
        client = Client()
        client.start()
        time.sleep(2) #wait for the DNS to see the client connected
        nbClientsConnected = self.dbr.getServerClients(SERVER_NAME)
        self.assertEqual(nbClientsConnected,1)
        self.assertIsNotNone(next(self.dbr.getNextServerClient()))
        client.stop()

if __name__ == "__main__":
    dns_node = os.getenv("DIM_DNS_NODE")
    #change the value of the DNS node if necessary by adding the line "dns_node = name_of_dns_node"
    if dns_node is not None:
        pydim.dis_set_dns_node(dns_node)

    if not pydim.dis_get_dns_node():
        print("No Dim DNS node found. Please set the environment variable DIM_DNS_NODE")
        sys.exit(1)

    loader = unittest.TestLoader()

    test_suite = unittest.TestSuite()

    test_suite.addTests(loader.loadTestsFromTestCase(DimBrowserGetServicesTestCase))
    test_suite.addTests(loader.loadTestsFromTestCase(DimBrowserGetServersTestCase))
    test_suite.addTests(loader.loadTestsFromTestCase(DimBrowserGetServerServicesTestCase))
    test_suite.addTests(loader.loadTestsFromTestCase(DimBrowserGetServerClientsTestCase))

    unittest.TextTestRunner(verbosity=1).run(test_suite)

'''from dimbrowser import DimBrowser
import sys
if __name__ == '__main__':
    dbr = DimBrowser()

    nbRes = dbr.getServices("example-command*")
    print("nbServices of example-command server = %s" % nbRes)
    for i in range(nbRes):
        print(dbr.getNextService().next())

    nbServers = dbr.getServers()
    print("NbServers = %s" % nbServers)
    for i in range(nbServers):
        print(dbr.getNextServer().next())

    nbServerServices = dbr.getServerServices("example-commands")
    print("NbServerServices of example-commands server %s" % nbServerServices)
    for i in range(nbServerServices-1):
        print(dbr.getNextServerService().next())

    nbClientsConnected = dbr.getServerClients("example-commands")
    print("nbClientsConnected to server-name = %s" % nbClientsConnected)
    for i in range(nbClientsConnected):
        print(dbr.getNextServerClient().next())

    del dbr'''
